@@ -114,7 +114,7 @@ bool FtpNegotiationCommand::executeInternal() {
     command->setStartupIdleTime(getOption()->getAsInt(PREF_STARTUP_IDLE_TIME));
     command->setLowestDownloadSpeedLimit
       (getOption()->getAsInt(PREF_LOWEST_SPEED_LIMIT));
-    if(!getFileEntry()->isSingleHostMultiConnectionEnabled()) {
+    if(getFileEntry()->isUniqueProtocol()) {
       getFileEntry()->removeURIWhoseHostnameIs(getRequest()->getHost());
     }
     getRequestGroup()->getURISelector()->tuneDownloadCommand
@@ -126,9 +126,9 @@ bool FtpNegotiationCommand::executeInternal() {
     return true;
   } else if(sequence_ == SEQ_FILE_PREPARATION) {
     if(getOption()->getAsBool(PREF_FTP_PASV)) {
-      sequence_ = SEQ_SEND_PASV;
+      sequence_ = SEQ_PREPARE_PASV;
     } else {
-      sequence_ = SEQ_PREPARE_SERVER_SOCKET;
+      sequence_ = SEQ_PREPARE_PORT;
     }
     return false;
   } else if(sequence_ == SEQ_EXIT) {
@@ -141,7 +141,8 @@ bool FtpNegotiationCommand::executeInternal() {
 
 bool FtpNegotiationCommand::recvGreeting() {
   if(!checkIfConnectionEstablished
-     (getSocket(), connectedHostname_, connectedAddr_, connectedPort_)) {
+     (getSocket(), getRequest()->getConnectedHostname(),
+      getRequest()->getConnectedAddr(), getRequest()->getConnectedPort())) {
     sequence_ = SEQ_EXIT;
     return false;
   }
@@ -259,14 +260,24 @@ bool FtpNegotiationCommand::recvPwd()
     getLogger()->info("CUID#%s - base working directory is '%s'",
                       util::itos(getCuid()).c_str(), pwd.c_str());
   }
+  sequence_ = SEQ_SEND_CWD_PREP;
+  return true;
+}
+
+bool FtpNegotiationCommand::sendCwdPrep()
+{
+  // Calling setReadCheckSocket() is needed when the socket is reused, 
+  setReadCheckSocket(getSocket());
+  util::split(getRequest()->getDir(), std::back_inserter(cwdDirs_),
+              A2STR::SLASH_C);
+  cwdDirs_.push_front(ftp_->getBaseWorkingDir());
   sequence_ = SEQ_SEND_CWD;
   return true;
 }
 
-bool FtpNegotiationCommand::sendCwd() {
-  // Calling setReadCheckSocket() is needed when the socket is reused, 
-  setReadCheckSocket(getSocket());
-  if(ftp_->sendCwd()) {
+bool FtpNegotiationCommand::sendCwd()
+{
+  if(ftp_->sendCwd(cwdDirs_.front())) {
     disableWriteCheckSocket();
     sequence_ = SEQ_RECV_CWD;
   } else {
@@ -275,7 +286,8 @@ bool FtpNegotiationCommand::sendCwd() {
   return false;
 }
 
-bool FtpNegotiationCommand::recvCwd() {
+bool FtpNegotiationCommand::recvCwd()
+{
   unsigned int status = ftp_->receiveResponse();
   if(status == 0) {
     return false;
@@ -289,10 +301,15 @@ bool FtpNegotiationCommand::recvCwd() {
     else
       throw DL_ABORT_EX(StringFormat(EX_BAD_STATUS, status).str());
   }
-  if(getOption()->getAsBool(PREF_REMOTE_TIME)) {
-    sequence_ = SEQ_SEND_MDTM;
+  cwdDirs_.pop_front();
+  if(cwdDirs_.empty()) {
+    if(getOption()->getAsBool(PREF_REMOTE_TIME)) {
+      sequence_ = SEQ_SEND_MDTM;
+    } else {
+      sequence_ = SEQ_SEND_SIZE;
+    }
   } else {
-    sequence_ = SEQ_SEND_SIZE;
+    sequence_ = SEQ_SEND_CWD;
   }
   return true;
 }
@@ -318,16 +335,9 @@ bool FtpNegotiationCommand::recvMdtm()
   if(status == 213) {
     if(lastModifiedTime.good()) {
       getRequestGroup()->updateLastModifiedTime(lastModifiedTime);
-      time_t t = lastModifiedTime.getTime();
-      struct tm* tms = gmtime(&t); // returned struct is statically allocated.
-      if(tms) {
-        if(getLogger()->debug()) {
-          getLogger()->debug("MDTM result was parsed as: %s GMT", asctime(tms));
-        }
-      } else {
-        if(getLogger()->debug()) {
-          getLogger()->debug("gmtime() failed for MDTM result.");
-        }
+      if(getLogger()->debug()) {
+        getLogger()->debug("MDTM result was parsed as: %s",
+                           lastModifiedTime.toHTTPDate().c_str());
       }
     } else {
       if(getLogger()->debug()) {
@@ -360,10 +370,9 @@ bool FtpNegotiationCommand::onFileSizeDetermined(uint64_t totalLength)
   getFileEntry()->setLength(totalLength);
   if(getFileEntry()->getPath().empty()) {
     getFileEntry()->setPath
-      (util::applyDir
+      (util::createSafePath
        (getDownloadContext()->getDir(),
-        util::fixTaintedBasename
-        (util::percentDecode(getRequest()->getFile()))));
+        util::percentDecode(getRequest()->getFile())));
   }
   getRequestGroup()->preDownloadProcessing();
   if(getDownloadEngine()->getRequestGroupMan()->
@@ -375,9 +384,9 @@ bool FtpNegotiationCommand::onFileSizeDetermined(uint64_t totalLength)
   if(totalLength == 0) {
 
     if(getOption()->getAsBool(PREF_FTP_PASV)) {
-      sequence_ = SEQ_SEND_PASV;
+      sequence_ = SEQ_PREPARE_PASV;
     } else {
-      sequence_ = SEQ_PREPARE_SERVER_SOCKET;
+      sequence_ = SEQ_PREPARE_PORT;
     }
 
     if(getOption()->getAsBool(PREF_DRY_RUN)) {
@@ -386,11 +395,17 @@ bool FtpNegotiationCommand::onFileSizeDetermined(uint64_t totalLength)
       return false;
     }
 
-    if(getRequestGroup()->downloadFinishedByFileLength()) {
+    if(getDownloadContext()->knowsTotalLength() &&
+       getRequestGroup()->downloadFinishedByFileLength()) {
+      // TODO If metalink file does not contain size and it contains
+      // hash and file is not zero length, but remote server says the
+      // file size is 0, no hash check is performed in the current
+      // implementation. See also
+      // HttpResponseCommand::handleOtherEncoding()
       getRequestGroup()->initPieceStorage();
       getPieceStorage()->markAllPiecesDone();
+      getDownloadContext()->setChecksumVerified(true);
       sequence_ = SEQ_DOWNLOAD_ALREADY_COMPLETED;
-
       getLogger()->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
                           util::itos(getRequestGroup()->getGID()).c_str(),
                           getRequestGroup()->getFirstFilePath().c_str());
@@ -406,13 +421,14 @@ bool FtpNegotiationCommand::onFileSizeDetermined(uint64_t totalLength)
 
     if(getDownloadContext()->knowsTotalLength()) {
       sequence_ = SEQ_DOWNLOAD_ALREADY_COMPLETED;
+      getPieceStorage()->markAllPiecesDone();
       poolConnection();
       return false;
     }
     // We have to make sure that command that has Request object must
     // have segment after PieceStorage is initialized. See
     // AbstractCommand::execute()
-    getSegmentMan()->getSegment(getCuid(), 0);
+    getSegmentMan()->getSegmentWithIndex(getCuid(), 0);
     return true;
   } else {
     getRequestGroup()->adjustFilename
@@ -428,31 +444,20 @@ bool FtpNegotiationCommand::onFileSizeDetermined(uint64_t totalLength)
       return false;
     }
 
-    BtProgressInfoFileHandle infoFile
-      (new DefaultBtProgressInfoFile(getDownloadContext(),
-                                     getPieceStorage(),
-                                     getOption().get()));
-    if(!infoFile->exists() &&
-       getRequestGroup()->downloadFinishedByFileLength()) {
-      getPieceStorage()->markAllPiecesDone();
-
+    SharedHandle<CheckIntegrityEntry> checkIntegrityEntry =
+      getRequestGroup()->createCheckIntegrityEntry();
+    if(checkIntegrityEntry.isNull()) {
       sequence_ = SEQ_DOWNLOAD_ALREADY_COMPLETED;
-      
-      getLogger()->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
-                          util::itos(getRequestGroup()->getGID()).c_str(),
-                          getRequestGroup()->getFirstFilePath().c_str());
-
       poolConnection();
-      
       return false;
     }
-    getRequestGroup()->loadAndOpenFile(infoFile);
+    checkIntegrityEntry->pushNextCommand(this);
     // We have to make sure that command that has Request object must
     // have segment after PieceStorage is initialized. See
     // AbstractCommand::execute()
-    getSegmentMan()->getSegment(getCuid(), 0);
+    getSegmentMan()->getSegmentWithIndex(getCuid(), 0);
 
-    prepareForNextAction(this);
+    prepareForNextAction(checkIntegrityEntry);
 
     disableReadCheckSocket();
   }
@@ -498,9 +503,9 @@ bool FtpNegotiationCommand::recvSize() {
     // wrong file to be downloaded if user-specified URL is wrong.
   }
   if(getOption()->getAsBool(PREF_FTP_PASV)) {
-    sequence_ = SEQ_SEND_PASV;
+    sequence_ = SEQ_PREPARE_PASV;
   } else {
-    sequence_ = SEQ_PREPARE_SERVER_SOCKET;
+    sequence_ = SEQ_PREPARE_PORT;
   }
   return true;
 }
@@ -510,6 +515,22 @@ void FtpNegotiationCommand::afterFileAllocation()
   setReadCheckSocket(getSocket());
 }
 
+bool FtpNegotiationCommand::preparePort() {
+  afterFileAllocation();
+  if(getSocket()->getAddressFamily() == AF_INET6) {
+    sequence_ = SEQ_PREPARE_SERVER_SOCKET_EPRT;
+  } else {
+    sequence_ = SEQ_PREPARE_SERVER_SOCKET;
+  }
+  return true;
+}
+
+bool FtpNegotiationCommand::prepareServerSocketEprt() {
+  serverSocket_ = ftp_->createServerSocket();
+  sequence_ = SEQ_SEND_EPRT;
+  return true;
+}
+
 bool FtpNegotiationCommand::prepareServerSocket()
 {
   serverSocket_ = ftp_->createServerSocket();
@@ -517,8 +538,30 @@ bool FtpNegotiationCommand::prepareServerSocket()
   return true;
 }
 
+bool FtpNegotiationCommand::sendEprt() {
+  if(ftp_->sendEprt(serverSocket_)) {
+    disableWriteCheckSocket();
+    sequence_ = SEQ_RECV_EPRT;
+  } else {
+    setWriteCheckSocket(getSocket());
+  }
+  return false;
+}
+
+bool FtpNegotiationCommand::recvEprt() {
+  unsigned int status = ftp_->receiveResponse();
+  if(status == 0) {
+    return false;
+  }
+  if(status == 200) {
+    sequence_ = SEQ_SEND_REST;
+  } else {
+    sequence_ = SEQ_PREPARE_SERVER_SOCKET;
+  }
+  return true;
+}
+
 bool FtpNegotiationCommand::sendPort() {
-  afterFileAllocation();
   if(ftp_->sendPort(serverSocket_)) {
     disableWriteCheckSocket();
     sequence_ = SEQ_RECV_PORT;
@@ -540,8 +583,45 @@ bool FtpNegotiationCommand::recvPort() {
   return true;
 }
 
-bool FtpNegotiationCommand::sendPasv() {
+bool FtpNegotiationCommand::preparePasv() {
   afterFileAllocation();
+  if(getSocket()->getAddressFamily() == AF_INET6) {
+    sequence_ = SEQ_SEND_EPSV;
+  } else {
+    sequence_ = SEQ_SEND_PASV;
+  }
+  return true;
+}
+
+bool FtpNegotiationCommand::sendEpsv() {
+  if(ftp_->sendEpsv()) {
+    disableWriteCheckSocket();
+    sequence_ = SEQ_RECV_EPSV;
+  } else {
+    setWriteCheckSocket(getSocket());
+  }
+  return true;
+}
+
+bool FtpNegotiationCommand::recvEpsv() {
+  uint16_t port;
+  unsigned int status = ftp_->receiveEpsvResponse(port);
+  if(status == 0) {
+    return false;
+  }
+  if(status == 229) {
+    std::pair<std::string, uint16_t> peerInfo;
+    getSocket()->getPeerInfo(peerInfo);
+    peerInfo.second = port;
+    dataConnAddr_ = peerInfo;
+    return preparePasvConnect();
+  } else {
+    sequence_ = SEQ_SEND_PASV;
+    return true;
+  }
+}
+
+bool FtpNegotiationCommand::sendPasv() {
   if(ftp_->sendPasv()) {
     disableWriteCheckSocket();
     sequence_ = SEQ_RECV_PASV;
@@ -560,20 +640,26 @@ bool FtpNegotiationCommand::recvPasv() {
   if(status != 227) {
     throw DL_ABORT_EX(StringFormat(EX_BAD_STATUS, status).str());
   }
-  // TODO Should we check to see that dest.first is not in noProxy list?
+  dataConnAddr_ = dest;
+
+  return preparePasvConnect();
+}
+
+bool FtpNegotiationCommand::preparePasvConnect() {
+  // TODO Should we check to see that dataConnAddr_.first is not in
+  // noProxy list?
   if(isProxyDefined()) {
-    dataConnAddr_ = dest;
     sequence_ = SEQ_RESOLVE_PROXY;
     return true;
   } else {
     // make a data connection to the server.
     if(getLogger()->info()) {
       getLogger()->info(MSG_CONNECTING_TO_SERVER, util::itos(getCuid()).c_str(),
-                        dest.first.c_str(),
-                        dest.second);
+                        dataConnAddr_.first.c_str(),
+                        dataConnAddr_.second);
     }
     dataSocket_.reset(new SocketCore());
-    dataSocket_->establishConnection(dest.first, dest.second);
+    dataSocket_->establishConnection(dataConnAddr_.first, dataConnAddr_.second);
     disableReadCheckSocket();
     setWriteCheckSocket(dataSocket_);
     sequence_ = SEQ_SEND_REST_PASV;
@@ -675,6 +761,12 @@ bool FtpNegotiationCommand::recvTunnelResponse()
 
 bool FtpNegotiationCommand::sendRestPasv(const SharedHandle<Segment>& segment) {
   //dataSocket_->setBlockingMode();
+  // Check connection is made properly
+  if(dataSocket_->isReadable(0)) {
+    std::string error = dataSocket_->getSocketError();
+    throw DL_ABORT_EX
+      (StringFormat(MSG_ESTABLISHING_CONNECTION_FAILED, error.c_str()).str());
+  }
   setReadCheckSocket(getSocket());
   disableWriteCheckSocket();
   return sendRest(segment);
@@ -773,6 +865,8 @@ bool FtpNegotiationCommand::processSequence
     return sendPwd();
   case SEQ_RECV_PWD:
     return recvPwd();
+  case SEQ_SEND_CWD_PREP:
+    return sendCwdPrep();
   case SEQ_SEND_CWD:
     return sendCwd();
   case SEQ_RECV_CWD:
@@ -785,12 +879,26 @@ bool FtpNegotiationCommand::processSequence
     return sendSize();
   case SEQ_RECV_SIZE:
     return recvSize();
+  case SEQ_PREPARE_PORT:
+    return preparePort();
+  case SEQ_PREPARE_SERVER_SOCKET_EPRT:
+    return prepareServerSocketEprt();
+  case SEQ_SEND_EPRT:
+    return sendEprt();
+  case SEQ_RECV_EPRT:
+    return recvEprt();
   case SEQ_PREPARE_SERVER_SOCKET:
     return prepareServerSocket();
   case SEQ_SEND_PORT:
     return sendPort();
   case SEQ_RECV_PORT:
     return recvPort();
+  case SEQ_PREPARE_PASV:
+    return preparePasv();
+  case SEQ_SEND_EPSV:
+    return sendEpsv();
+  case SEQ_RECV_EPSV:
+    return recvEpsv();
   case SEQ_SEND_PASV:
     return sendPasv();
   case SEQ_RECV_PASV:
@@ -832,6 +940,7 @@ void FtpNegotiationCommand::poolConnection() const
 void FtpNegotiationCommand::onDryRunFileFound()
 {
   getPieceStorage()->markAllPiecesDone();
+  getDownloadContext()->setChecksumVerified(true);
   poolConnection();
   sequence_ = SEQ_HEAD_OK;
 }
