@@ -44,7 +44,7 @@
 namespace aria2 {
 
 FtpTunnelResponseCommand::FtpTunnelResponseCommand
-(int cuid,
+(cuid_t cuid,
  const SharedHandle<Request>& req,
  const SharedHandle<FileEntry>& fileEntry,
  RequestGroup* requestGroup,
@@ -59,8 +59,9 @@ FtpTunnelResponseCommand::~FtpTunnelResponseCommand() {}
 
 Command* FtpTunnelResponseCommand::getNextCommand()
 {
-  return new FtpNegotiationCommand(cuid, req, _fileEntry,
-                                   _requestGroup, e, socket);
+  return new FtpNegotiationCommand(getCuid(), getRequest(), getFileEntry(),
+                                   getRequestGroup(), getDownloadEngine(),
+                                   getSocket());
 }
 
 } // namespace aria2
