@@ -47,7 +47,14 @@
 #include "HttpHeader.h"
 #include "Range.h"
 #include "DownloadContext.h"
-#include "Decoder.h"
+#include "RequestGroupMan.h"
+#include "FileAllocationEntry.h"
+#include "CheckIntegrityEntry.h"
+#include "ServerStatMan.h"
+#include "Logger.h"
+#include "StreamFilter.h"
+#include "SinkStreamFilter.h"
+#include "util.h"
 
 namespace aria2 {
 
@@ -61,60 +68,63 @@ HttpDownloadCommand::HttpDownloadCommand
  DownloadEngine* e,
  const SocketHandle& socket)
   :DownloadCommand(cuid, req, fileEntry, requestGroup, e, socket),
-   _httpResponse(httpResponse),
-   _httpConnection(httpConnection) {}
+   httpResponse_(httpResponse),
+   httpConnection_(httpConnection) {}
 
 HttpDownloadCommand::~HttpDownloadCommand() {}
 
 bool HttpDownloadCommand::prepareForNextSegment() {
-  bool downloadFinished = _requestGroup->downloadFinished();
-  if(req->isPipeliningEnabled() && !downloadFinished) {
+  bool downloadFinished = getRequestGroup()->downloadFinished();
+  if(getRequest()->isPipeliningEnabled() && !downloadFinished) {
     HttpRequestCommand* command =
-      new HttpRequestCommand(cuid, req, _fileEntry,
-                             _requestGroup, _httpConnection, e,
-                             socket);
+      new HttpRequestCommand(getCuid(), getRequest(), getFileEntry(),
+                             getRequestGroup(), httpConnection_,
+                             getDownloadEngine(), getSocket());
     // Set proxy request here. aria2 sends the HTTP request specialized for
     // proxy.
-    if(resolveProxyMethod(req->getProtocol()) == V_GET) {
+    if(resolveProxyMethod(getRequest()->getProtocol()) == V_GET) {
       command->setProxyRequest(createProxyRequest());
     }
-    e->commands.push_back(command);
+    getDownloadEngine()->addCommand(command);
     return true;
   } else {
-    if(req->isPipeliningEnabled() ||
-       (req->isKeepAliveEnabled() &&
+    if(getRequest()->isPipeliningEnabled() ||
+       (getRequest()->isKeepAliveEnabled() &&
         (
-         ((!_transferEncodingDecoder.isNull() &&
-           _transferEncodingDecoder->finished()) ||
-          (!_contentEncodingDecoder.isNull() &&
-           _contentEncodingDecoder->finished())) ||
-         _fileEntry->getLastOffset() ==
-         _segments.front()->getPositionToWrite()
+         // Make sure that all filters are finished to pool socket
+         (!util::endsWith(getStreamFilter()->getName(),
+                          SinkStreamFilter::NAME) &&
+          getStreamFilter()->finished()) ||
+         getRequestEndOffset() ==
+         getFileEntry()->gtoloff(getSegments().front()->getPositionToWrite())
          )
         )
        ) {
-      // TODO What if server sends EOF when _contentEncodingDecoder is
+      // TODO What if server sends EOF when non-SinkStreamFilter is
       // used and server didn't send Connection: close? We end up to
       // pool terminated socket.  In HTTP/1.1, keep-alive is default,
       // so closing connection without Connection: close header means
       // that server is broken or not configured properly.
-      e->poolSocket(req, createProxyRequest(), socket);
+      getDownloadEngine()->poolSocket
+        (getRequest(), createProxyRequest(), getSocket());
     }
     // The request was sent assuming that server supported pipelining, but
     // it turned out that server didn't support it.
     // We detect this situation by comparing the end byte in range header
     // of the response with the end byte of segment.
     // If it is the same, HTTP negotiation is necessary for the next request.
-    if(!req->isPipeliningEnabled() && req->isPipeliningHint() &&
+    if(!getRequest()->isPipeliningEnabled() &&
+       getRequest()->isPipeliningHint() &&
        !downloadFinished) {
-      const SharedHandle<Segment>& segment = _segments.front();
+      const SharedHandle<Segment>& segment = getSegments().front();
 
-      off_t lastOffset =_fileEntry->gtoloff
-        (std::min(static_cast<off_t>(segment->getPosition()+segment->getLength()),
-                  _fileEntry->getLastOffset()));
+      off_t lastOffset =getFileEntry()->gtoloff
+        (std::min(static_cast<off_t>
+                  (segment->getPosition()+segment->getLength()),
+                  getFileEntry()->getLastOffset()));
       
       if(lastOffset ==
-         _httpResponse->getHttpHeader()->getRange()->getEndByte()+1) {
+         httpResponse_->getHttpHeader()->getRange()->getEndByte()+1) {
         return prepareForRetry(0);
       }
     }
@@ -122,4 +132,14 @@ bool HttpDownloadCommand::prepareForNextSegment() {
   }
 }
 
+off_t HttpDownloadCommand::getRequestEndOffset() const
+{
+  off_t endByte = httpResponse_->getHttpHeader()->getRange()->getEndByte();
+  if(endByte > 0) {
+    return endByte+1;
+  } else {
+    return endByte;
+  }
+}
+
 } // namespace aria2
