@@ -49,7 +49,10 @@
 #include "A2STR.h"
 #include "DownloadContext.h"
 #include "util.h"
-
+#include "RequestGroupMan.h"
+#include "FileAllocationEntry.h"
+#include "CheckIntegrityEntry.h"
+#include "ServerStatMan.h"
 namespace aria2 {
 
 HttpInitiateConnectionCommand::HttpInitiateConnectionCommand
@@ -70,33 +73,39 @@ Command* HttpInitiateConnectionCommand::createNextCommand
   Command* command;
   if(!proxyRequest.isNull()) {
     SharedHandle<SocketCore> pooledSocket =
-      e->popPooledSocket(req->getHost(), req->getPort(),
-                         proxyRequest->getHost(), proxyRequest->getPort());
-    std::string proxyMethod = resolveProxyMethod(req->getProtocol());
+      getDownloadEngine()->popPooledSocket
+      (getRequest()->getHost(), getRequest()->getPort(),
+       proxyRequest->getHost(), proxyRequest->getPort());
+    std::string proxyMethod = resolveProxyMethod(getRequest()->getProtocol());
     if(pooledSocket.isNull()) {
-      if(logger->info()) {
-        logger->info(MSG_CONNECTING_TO_SERVER,
-                     util::itos(cuid).c_str(), addr.c_str(), port);
+      if(getLogger()->info()) {
+        getLogger()->info(MSG_CONNECTING_TO_SERVER,
+                          util::itos(getCuid()).c_str(), addr.c_str(), port);
       }
-      socket.reset(new SocketCore());
-      socket->establishConnection(addr, port);
+      createSocket();
+      getSocket()->establishConnection(addr, port);
 
+      getRequest()->setConnectedAddrInfo(hostname, addr, port);
       if(proxyMethod == V_TUNNEL) {
         HttpProxyRequestCommand* c =
-          new HttpProxyRequestCommand(cuid, req, _fileEntry,
-                                      _requestGroup, e,
-                                      proxyRequest, socket);
-        c->setConnectedAddr(hostname, addr, port);
+          new HttpProxyRequestCommand(getCuid(),
+                                      getRequest(),
+                                      getFileEntry(),
+                                      getRequestGroup(),
+                                      getDownloadEngine(),
+                                      proxyRequest,
+                                      getSocket());
         command = c;
       } else if(proxyMethod == V_GET) {
         SharedHandle<HttpConnection> httpConnection
-          (new HttpConnection(cuid, socket, getOption().get()));
-        HttpRequestCommand* c = new HttpRequestCommand(cuid, req,
-                                                       _fileEntry,
-                                                       _requestGroup,
-                                                       httpConnection, e,
-                                                       socket);
-        c->setConnectedAddr(hostname, addr, port);
+          (new HttpConnection(getCuid(), getSocket()));
+        HttpRequestCommand* c = new HttpRequestCommand(getCuid(),
+                                                       getRequest(),
+                                                       getFileEntry(),
+                                                       getRequestGroup(),
+                                                       httpConnection,
+                                                       getDownloadEngine(),
+                                                       getSocket());
         c->setProxyRequest(proxyRequest);
         command = c;
       } else {
@@ -104,12 +113,15 @@ Command* HttpInitiateConnectionCommand::createNextCommand
         throw DL_ABORT_EX("ERROR");
       }
     } else {
+      setConnectedAddrInfo(getRequest(), hostname, pooledSocket);
       SharedHandle<HttpConnection> httpConnection
-        (new HttpConnection(cuid, pooledSocket, getOption().get()));
-      HttpRequestCommand* c = new HttpRequestCommand(cuid, req,
-                                                     _fileEntry,
-                                                     _requestGroup,
-                                                     httpConnection, e,
+        (new HttpConnection(getCuid(), pooledSocket));
+      HttpRequestCommand* c = new HttpRequestCommand(getCuid(),
+                                                     getRequest(),
+                                                     getFileEntry(),
+                                                     getRequestGroup(),
+                                                     httpConnection,
+                                                     getDownloadEngine(),
                                                      pooledSocket);
       if(proxyMethod == V_GET) {
         c->setProxyRequest(proxyRequest);
@@ -118,24 +130,28 @@ Command* HttpInitiateConnectionCommand::createNextCommand
     }
   } else {
     SharedHandle<SocketCore> pooledSocket =
-      e->popPooledSocket(resolvedAddresses, req->getPort());
+      getDownloadEngine()->popPooledSocket
+      (resolvedAddresses, getRequest()->getPort());
     if(pooledSocket.isNull()) {
-      if(logger->info()) {
-        logger->info(MSG_CONNECTING_TO_SERVER,
-                     util::itos(cuid).c_str(), addr.c_str(), port);
+      if(getLogger()->info()) {
+        getLogger()->info(MSG_CONNECTING_TO_SERVER,
+                          util::itos(getCuid()).c_str(), addr.c_str(), port);
       }
-      socket.reset(new SocketCore());
-      socket->establishConnection(addr, port);
+      createSocket();
+      getSocket()->establishConnection(addr, port);
+      getRequest()->setConnectedAddrInfo(hostname, addr, port);
     } else {
-      socket = pooledSocket;
+      setSocket(pooledSocket);
+      setConnectedAddrInfo(getRequest(), hostname, pooledSocket);
     }
-    SharedHandle<HttpConnection> httpConnection(new HttpConnection(cuid, socket, getOption().get()));
+    SharedHandle<HttpConnection> httpConnection
+      (new HttpConnection(getCuid(), getSocket()));
     HttpRequestCommand* c =
-      new HttpRequestCommand(cuid, req, _fileEntry, _requestGroup,
-                             httpConnection, e, socket);
-    if(pooledSocket.isNull()) {
-      c->setConnectedAddr(hostname, addr, port);
-    }
+      new HttpRequestCommand(getCuid(), getRequest(), getFileEntry(),
+                             getRequestGroup(),
+                             httpConnection,
+                             getDownloadEngine(),
+                             getSocket());
     command = c;
   }
   return command;
