@@ -187,6 +187,7 @@ static void extractFileEntries
  const std::vector<std::string>& urlList)
 {
   std::string name;
+  std::string utf8Name;
   if(overrideName.empty()) {
     std::string nameKey;
     if(infoDict->containsKey(C_NAME_UTF8)) {
@@ -196,17 +197,18 @@ static void extractFileEntries
     }
     const String* nameData = asString(infoDict->get(nameKey));
     if(nameData) {
-      if(util::detectDirTraversal(nameData->s())) {
+      utf8Name = util::encodeNonUtf8(nameData->s());
+      if(util::detectDirTraversal(utf8Name)) {
         throw DL_ABORT_EX
           (StringFormat
            (MSG_DIR_TRAVERSAL_DETECTED,nameData->s().c_str()).str());
       }
       name = nameData->s();
     } else {
-      name = strconcat(File(defaultName).getBasename(), ".file");
+      name = utf8Name = strconcat(File(defaultName).getBasename(), ".file");
     }
   } else {
-    name = overrideName;
+    name = utf8Name = overrideName;
   }
   torrent->name = name;
   std::vector<SharedHandle<FileEntry> > fileEntries;
@@ -255,9 +257,11 @@ static void extractFileEntries
         }
       }
       std::string path = strjoin(pathelem.begin(), pathelem.end(), '/');
-      if(util::detectDirTraversal(path)) {
+      std::string utf8Path = strjoin(pathelem.begin(), pathelem.end(), '/',
+                                     std::ptr_fun(util::encodeNonUtf8));
+      if(util::detectDirTraversal(utf8Path)) {
         throw DL_ABORT_EX
-          (StringFormat(MSG_DIR_TRAVERSAL_DETECTED, path.c_str()).str());
+          (StringFormat(MSG_DIR_TRAVERSAL_DETECTED, utf8Path.c_str()).str());
       }
       std::string pePath =
         strjoin(pathelem.begin(), pathelem.end(), '/',
@@ -266,9 +270,8 @@ static void extractFileEntries
       std::vector<std::string> uris;
       createUri(urlList.begin(), urlList.end(),std::back_inserter(uris),pePath);
       SharedHandle<FileEntry> fileEntry
-        (new FileEntry(util::applyDir(ctx->getDir(), util::escapePath(path)),
-                       fileLengthData->i(),
-                       offset, uris));
+        (new FileEntry(util::applyDir(ctx->getDir(),util::escapePath(utf8Path)),
+                       fileLengthData->i(), offset, uris));
       fileEntry->setOriginalName(path);
       fileEntries.push_back(fileEntry);
       offset += fileEntry->getLength();
@@ -294,17 +297,15 @@ static void extractFileEntries
         uris.push_back(*i);
       }
     }
-
     SharedHandle<FileEntry> fileEntry
-      (new FileEntry(util::applyDir(ctx->getDir(), util::escapePath(name)),
-                     totalLength, 0,
-                     uris));
+      (new FileEntry(util::applyDir(ctx->getDir(), util::escapePath(utf8Name)),
+                     totalLength, 0, uris));
     fileEntry->setOriginalName(name);
     fileEntries.push_back(fileEntry);
   }
   ctx->setFileEntries(fileEntries.begin(), fileEntries.end());
   if(torrent->mode == MULTI) {
-    ctx->setBasePath(util::applyDir(ctx->getDir(), name));
+    ctx->setBasePath(util::applyDir(ctx->getDir(), utf8Name));
   }
 }
 
@@ -324,7 +325,7 @@ static void extractAnnounce
             eoi2 = tier->end(); uriIter != eoi2; ++uriIter) {
         const String* uri = asString(*uriIter);
         if(uri) {
-          ntier.push_back(util::trim(uri->s()));
+          ntier.push_back(util::strip(uri->s()));
         }
       }
       if(!ntier.empty()) {
@@ -335,7 +336,7 @@ static void extractAnnounce
     const String* announce = asString(rootDict->get(C_ANNOUNCE));
     if(announce) {
       std::vector<std::string> tier;
-      tier.push_back(util::trim(announce->s()));
+      tier.push_back(util::strip(announce->s()));
       torrent->announceList.push_back(tier);
     }
   }
@@ -356,7 +357,7 @@ static void extractNodes
       if(!hostname) {
         continue;
       }
-      if(util::trim(hostname->s()).empty()) {
+      if(util::strip(hostname->s()).empty()) {
         continue;
       }
       const Integer* port = asInteger(addrPairList->get(1));
@@ -576,14 +577,8 @@ void print(std::ostream& o, const SharedHandle<DownloadContext>& dctx)
     o << "Comment: " << torrentAttrs->comment << "\n";
   }
   if(torrentAttrs->creationDate) {
-    struct tm* staticNowtmPtr;
-    char buf[26];
-    time_t t = torrentAttrs->creationDate;
-    if((staticNowtmPtr = localtime(&t)) != 0 &&
-       asctime_r(staticNowtmPtr, buf) != 0) {
-      // buf includes "\n"
-      o << "Creation Date: " << buf;
-    }
+    o << "Creation Date: " << Time(torrentAttrs->creationDate).toHTTPDate()
+      << std::endl;
   }
   if(!torrentAttrs->createdBy.empty()) {
     o << "Created By: " << torrentAttrs->createdBy << "\n";
@@ -636,11 +631,11 @@ void computeFastSet
   }
   unsigned char tx[24];
   memcpy(tx, compact, 4);
-  if((tx[0] & 0x80) == 0 || (tx[0] & 0x40) == 0) {
-    tx[2] = 0x00;
-    tx[3] = 0x00;
+  if((tx[0] & 0x80u) == 0 || (tx[0] & 0x40u) == 0) {
+    tx[2] = 0x00u;
+    tx[3] = 0x00u;
   } else {
-    tx[3] = 0x00;
+    tx[3] = 0x00u;
   }
   memcpy(tx+4, infoHash, 20);
   unsigned char x[20];
@@ -837,8 +832,9 @@ int packcompact
 std::pair<std::string, uint16_t> unpackcompact
 (const unsigned char* compact, int family)
 {
-  int portOffset = family == AF_INET?4:16;
   std::pair<std::string, uint16_t> r;
+#ifdef HAVE_INET_NTOP
+  int portOffset = family == AF_INET?4:16;
   char buf[INET6_ADDRSTRLEN];
   if(!inet_ntop(family, compact, buf, sizeof(buf))) {
     return r;
@@ -847,6 +843,17 @@ std::pair<std::string, uint16_t> unpackcompact
   uint16_t portN;
   memcpy(&portN, compact+portOffset, sizeof(portN));
   r.second = ntohs(portN);
+#else // !HAVE_INET_NTOP
+  if(family != AF_INET) {
+    return r;
+  }
+  struct in_addr addr;
+  memcpy(&addr, compact, sizeof(in_addr));
+  r.first = inet_ntoa(addr);
+  uint16_t portN;
+  memcpy(&portN, compact+4, sizeof(portN));
+  r.second = ntohs(portN);
+#endif // !HAVE_INET_NTOP
   return r;
 }
 
