@@ -15,6 +15,7 @@
 #include "CookieStorage.h"
 #include "util.h"
 #include "AuthConfig.h"
+#include "TestUtil.h"
 
 namespace aria2 {
 
@@ -29,6 +30,7 @@ class HttpRequestTest : public CppUnit::TestFixture {
   CPPUNIT_TEST(testCreateRequest_query);
   CPPUNIT_TEST(testCreateRequest_head);
   CPPUNIT_TEST(testCreateRequest_ipv6LiteralAddr);
+  CPPUNIT_TEST(testCreateRequest_endOffsetOverride);
   CPPUNIT_TEST(testCreateProxyRequest);
   CPPUNIT_TEST(testIsRangeSatisfied);
   CPPUNIT_TEST(testUserAgent);
@@ -43,7 +45,7 @@ public:
   void setUp()
   {
     option_.reset(new Option());
-    option_->put(PREF_HTTP_AUTH_CHALLENGE, V_TRUE);
+    option_->put(PREF_HTTP_AUTH_CHALLENGE, A2_V_TRUE);
     authConfigFactory_.reset(new AuthConfigFactory());
   }
 
@@ -55,6 +57,7 @@ public:
   void testCreateRequest_query();
   void testCreateRequest_head();
   void testCreateRequest_ipv6LiteralAddr();
+  void testCreateRequest_endOffsetOverride();
   void testCreateProxyRequest();
   void testIsRangeSatisfied();
   void testUserAgent();
@@ -378,16 +381,19 @@ void HttpRequestTest::testCreateRequest_with_cookie()
     (new PiecedSegment(1024*1024, p));
   SharedHandle<FileEntry> fileEntry(new FileEntry("file", 1024*1024*10, 0));
 
-  Cookie cookie1("name1", "value1", "/archives", "localhost", false);
-  Cookie cookie2("name2", "value2", "/archives/download", "localhost", false);
-  Cookie cookie3("name3", "value3", "/archives/download", ".aria2.org", false);
-  Cookie cookie4("name4", "value4", "/archives/", ".aria2.org", true);
-
+  Cookie cookies[] = {
+    createCookie("name1", "value1", "localhost", true, "/archives", false),
+    createCookie("name2", "value2", "localhost", true,
+                 "/archives/download",  false),
+    createCookie("name3", "value3", "aria2.org", false,
+                 "/archives/download",  false),
+    createCookie("name4", "value4", "aria2.org", false, "/archives/", true),
+    createCookie("name5", "value5", "example.org", false, "/", false)
+  };
   SharedHandle<CookieStorage> st(new CookieStorage());
-  CPPUNIT_ASSERT(st->store(cookie1));
-  CPPUNIT_ASSERT(st->store(cookie2));
-  CPPUNIT_ASSERT(st->store(cookie3));
-  CPPUNIT_ASSERT(st->store(cookie4));
+  for(size_t i = 0; i < A2_ARRAY_LEN(cookies); ++i) {
+    CPPUNIT_ASSERT(st->store(cookies[i], 0));
+  }
 
   HttpRequest httpRequest;
 
@@ -452,7 +458,35 @@ void HttpRequestTest::testCreateRequest_with_cookie()
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
-  
+
+  // The path of cookie4 ends with '/'
+  request->setUri("https://www.aria2.org/archives/aria2-1.0.0.tar.bz2");
+
+  expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Accept: */*\r\n"
+    "Host: www.aria2.org\r\n"
+    "Pragma: no-cache\r\n"
+    "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
+    "Cookie: name4=value4;\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());  
+
+  request->setUri("http://example.org/aria2-1.0.0.tar.bz2");
+
+  expectedText = "GET /aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Accept: */*\r\n"
+    "Host: example.org\r\n"
+    "Pragma: no-cache\r\n"
+    "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
+    "Cookie: name5=value5;\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 }
 
 void HttpRequestTest::testCreateRequest_query()
@@ -490,10 +524,54 @@ void HttpRequestTest::testCreateRequest_head()
   std::stringstream result(httpRequest.createRequest());
   std::string line;
   CPPUNIT_ASSERT(getline(result, line));
-  util::trimSelf(line);
+  line = util::strip(line);
   CPPUNIT_ASSERT_EQUAL(std::string("HEAD /aria2-1.0.0.tar.bz2 HTTP/1.1"), line);
 }
 
+void HttpRequestTest::testCreateRequest_endOffsetOverride()
+{
+  SharedHandle<Request> request(new Request());
+  request->setUri("http://localhost/myfile");
+  HttpRequest httpRequest;
+  httpRequest.disableContentEncoding();
+  httpRequest.setRequest(request);
+  httpRequest.setAuthConfigFactory(authConfigFactory_, option_.get());
+  SharedHandle<Piece> p(new Piece(0, 1024*1024));
+  SharedHandle<Segment> segment(new PiecedSegment(1024*1024, p));
+  httpRequest.setSegment(segment);
+  httpRequest.setEndOffsetOverride(1024*1024*1024);
+  SharedHandle<FileEntry> fileEntry
+    (new FileEntry("file", 1024*1024*1024*10LL, 0));
+  httpRequest.setFileEntry(fileEntry);
+  // End byte is not passed if start byte is 0.
+  std::string expectedText =
+    "GET /myfile HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Accept: */*\r\n"
+    "Host: localhost\r\n"
+    "Pragma: no-cache\r\n"
+    "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
+    "\r\n";
+  
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
+
+  segment->updateWrittenLength(1);
+
+  expectedText =
+    "GET /myfile HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Accept: */*\r\n"
+    "Host: localhost\r\n"
+    "Pragma: no-cache\r\n"
+    "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
+    "Range: bytes=1-1073741823\r\n"
+    "\r\n";
+  
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
+}
+
 void HttpRequestTest::testCreateProxyRequest()
 {
   SharedHandle<Request> request(new Request());
