@@ -26,6 +26,8 @@ class OptionHandlerTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testFloatNumberOptionHandler_max);
   CPPUNIT_TEST(testFloatNumberOptionHandler_min_max);
   CPPUNIT_TEST(testHttpProxyOptionHandler);
+  CPPUNIT_TEST(testHttpProxyUserOptionHandler);
+  CPPUNIT_TEST(testHttpProxyPasswdOptionHandler);
   CPPUNIT_TEST_SUITE_END();
   
 public:
@@ -45,6 +47,8 @@ public:
   void testFloatNumberOptionHandler_max();
   void testFloatNumberOptionHandler_min_max();
   void testHttpProxyOptionHandler();
+  void testHttpProxyUserOptionHandler();
+  void testHttpProxyPasswdOptionHandler();
 };
 
 
@@ -65,16 +69,16 @@ void OptionHandlerTest::testBooleanOptionHandler()
   CPPUNIT_ASSERT(handler.canHandle("foo"));
   CPPUNIT_ASSERT(!handler.canHandle("foobar"));
   Option option;
-  handler.parse(option, V_TRUE);
-  CPPUNIT_ASSERT_EQUAL(std::string(V_TRUE), option.get("foo"));
-  handler.parse(option, V_FALSE);
-  CPPUNIT_ASSERT_EQUAL(std::string(V_FALSE), option.get("foo"));
+  handler.parse(option, A2_V_TRUE);
+  CPPUNIT_ASSERT_EQUAL(std::string(A2_V_TRUE), option.get("foo"));
+  handler.parse(option, A2_V_FALSE);
+  CPPUNIT_ASSERT_EQUAL(std::string(A2_V_FALSE), option.get("foo"));
   try {
     handler.parse(option, "hello");
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {}
   CPPUNIT_ASSERT_EQUAL(std::string("true,false"),
-		       handler.createPossibleValuesString());
+                       handler.createPossibleValuesString());
 }
 
 void OptionHandlerTest::testNumberOptionHandler()
@@ -86,7 +90,7 @@ void OptionHandlerTest::testNumberOptionHandler()
   handler.parse(option, "0");
   CPPUNIT_ASSERT_EQUAL(std::string("0"), option.get("foo"));
   CPPUNIT_ASSERT_EQUAL(std::string("*-*"),
-		       handler.createPossibleValuesString());
+                       handler.createPossibleValuesString());
 }
 
 void OptionHandlerTest::testNumberOptionHandler_min()
@@ -100,7 +104,7 @@ void OptionHandlerTest::testNumberOptionHandler_min()
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {}
   CPPUNIT_ASSERT_EQUAL(std::string("1-*"),
-		       handler.createPossibleValuesString());
+                       handler.createPossibleValuesString());
 }
 
 void OptionHandlerTest::testNumberOptionHandler_max()
@@ -114,7 +118,7 @@ void OptionHandlerTest::testNumberOptionHandler_max()
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {}
   CPPUNIT_ASSERT_EQUAL(std::string("*-100"),
-		       handler.createPossibleValuesString());
+                       handler.createPossibleValuesString());
 }
 
 void OptionHandlerTest::testNumberOptionHandler_min_max()
@@ -134,7 +138,7 @@ void OptionHandlerTest::testNumberOptionHandler_min_max()
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {}
   CPPUNIT_ASSERT_EQUAL(std::string("1-100"),
-		       handler.createPossibleValuesString());
+                       handler.createPossibleValuesString());
 }
 
 void OptionHandlerTest::testUnitNumberOptionHandler()
@@ -175,7 +179,7 @@ void OptionHandlerTest::testParameterOptionHandler_1argInit()
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {}
   CPPUNIT_ASSERT_EQUAL(std::string("value1"),
-		       handler.createPossibleValuesString());
+                       handler.createPossibleValuesString());
 }
 
 void OptionHandlerTest::testParameterOptionHandler_2argsInit()
@@ -193,12 +197,12 @@ void OptionHandlerTest::testParameterOptionHandler_2argsInit()
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {}
   CPPUNIT_ASSERT_EQUAL(std::string("value1,value2"),
-		       handler.createPossibleValuesString());
+                       handler.createPossibleValuesString());
 }
 
 void OptionHandlerTest::testParameterOptionHandler_listInit()
 {
-  std::deque<std::string> validValues;
+  std::vector<std::string> validValues;
   validValues.push_back("value1");
   validValues.push_back("value2");
 
@@ -215,7 +219,7 @@ void OptionHandlerTest::testParameterOptionHandler_listInit()
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {}
   CPPUNIT_ASSERT_EQUAL(std::string("value1,value2"),
-		       handler.createPossibleValuesString());
+                       handler.createPossibleValuesString());
 }
 
 void OptionHandlerTest::testDefaultOptionHandler()
@@ -233,7 +237,7 @@ void OptionHandlerTest::testDefaultOptionHandler()
   handler.addTag("apple");
   CPPUNIT_ASSERT_EQUAL(std::string("apple"), handler.toTagString());
   handler.addTag("orange");
-  CPPUNIT_ASSERT_EQUAL(std::string("apple,orange"), handler.toTagString());
+  CPPUNIT_ASSERT_EQUAL(std::string("apple, orange"), handler.toTagString());
   CPPUNIT_ASSERT(handler.hasTag("apple"));
   CPPUNIT_ASSERT(handler.hasTag("orange"));
   CPPUNIT_ASSERT(!handler.hasTag("pineapple"));
@@ -248,7 +252,7 @@ void OptionHandlerTest::testFloatNumberOptionHandler()
   handler.parse(option, "1.0");
   CPPUNIT_ASSERT_EQUAL(std::string("1.0"), option.get("foo"));
   CPPUNIT_ASSERT_EQUAL(std::string("*-*"),
-		       handler.createPossibleValuesString());
+                       handler.createPossibleValuesString());
 }
 
 void OptionHandlerTest::testFloatNumberOptionHandler_min()
@@ -262,7 +266,7 @@ void OptionHandlerTest::testFloatNumberOptionHandler_min()
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {}
   CPPUNIT_ASSERT_EQUAL(std::string("0.0-*"),
-		       handler.createPossibleValuesString());
+                       handler.createPossibleValuesString());
 }
 
 void OptionHandlerTest::testFloatNumberOptionHandler_max()
@@ -276,7 +280,7 @@ void OptionHandlerTest::testFloatNumberOptionHandler_max()
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {}
   CPPUNIT_ASSERT_EQUAL(std::string("*-10.0"),
-		       handler.createPossibleValuesString());
+                       handler.createPossibleValuesString());
 }
 
 void OptionHandlerTest::testFloatNumberOptionHandler_min_max()
@@ -296,7 +300,7 @@ void OptionHandlerTest::testFloatNumberOptionHandler_min_max()
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {}
   CPPUNIT_ASSERT_EQUAL(std::string("0.0-10.0"),
-		       handler.createPossibleValuesString());
+                       handler.createPossibleValuesString());
 }
 
 void OptionHandlerTest::testHttpProxyOptionHandler()
@@ -307,18 +311,112 @@ void OptionHandlerTest::testHttpProxyOptionHandler()
   Option option;
   handler.parse(option, "proxy:65535");
   CPPUNIT_ASSERT_EQUAL(std::string("http://proxy:65535"),
-		       option.get(PREF_HTTP_PROXY));
+                       option.get(PREF_HTTP_PROXY));
 
   handler.parse(option, "http://proxy:8080");
   CPPUNIT_ASSERT_EQUAL(std::string("http://proxy:8080"),
-  		       option.get(PREF_HTTP_PROXY));
+                       option.get(PREF_HTTP_PROXY));
+
+  handler.parse(option, "");
+  CPPUNIT_ASSERT(option.defined(PREF_HTTP_PROXY));
+  CPPUNIT_ASSERT(option.blank(PREF_HTTP_PROXY));
 
   try {
     handler.parse(option, "http://bar:65536");
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {}
   CPPUNIT_ASSERT_EQUAL(std::string("[http://][USER:PASSWORD@]HOST[:PORT]"),
-		       handler.createPossibleValuesString());
+                       handler.createPossibleValuesString());
+
+  handler.parse(option, "http://user%40:passwd%40@proxy:8080");
+  CPPUNIT_ASSERT_EQUAL(std::string("http://user%40:passwd%40@proxy:8080"),
+                       option.get(PREF_HTTP_PROXY));
+
+  option.put(PREF_HTTP_PROXY_USER, "proxy@user");
+  handler.parse(option, "http://proxy:8080");
+  CPPUNIT_ASSERT_EQUAL(std::string("http://proxy%40user@proxy:8080"),
+                       option.get(PREF_HTTP_PROXY));
+
+  option.put(PREF_HTTP_PROXY_PASSWD, "proxy@passwd");
+  handler.parse(option, "http://proxy:8080");
+  CPPUNIT_ASSERT_EQUAL
+    (std::string("http://proxy%40user:proxy%40passwd@proxy:8080"),
+     option.get(PREF_HTTP_PROXY));
+
+  handler.parse(option, "http://user:passwd@proxy:8080");
+  CPPUNIT_ASSERT_EQUAL(std::string("http://user:passwd@proxy:8080"),
+                       option.get(PREF_HTTP_PROXY));
+
+  option.put(PREF_HTTP_PROXY_USER, "");
+  handler.parse(option, "http://proxy:8080");
+  CPPUNIT_ASSERT_EQUAL(std::string("http://:proxy%40passwd@proxy:8080"),
+                       option.get(PREF_HTTP_PROXY));
+
+  option.put(PREF_HTTP_PROXY_PASSWD, "");
+  handler.parse(option, "http://proxy:8080");
+  CPPUNIT_ASSERT_EQUAL(std::string("http://:@proxy:8080"),
+                       option.get(PREF_HTTP_PROXY));
+}
+
+void OptionHandlerTest::testHttpProxyUserOptionHandler()
+{
+  HttpProxyUserOptionHandler handler(PREF_HTTP_PROXY_USER, "", "");
+  Option option;
+  handler.parse(option, "proxyuser");
+  CPPUNIT_ASSERT_EQUAL(std::string("proxyuser"),
+                       option.get(PREF_HTTP_PROXY_USER));
+
+  option.put(PREF_HTTP_PROXY, "http://proxy:8080");
+  handler.parse(option, "proxy@user");
+  CPPUNIT_ASSERT_EQUAL(std::string("proxy@user"),
+                       option.get(PREF_HTTP_PROXY_USER));
+  CPPUNIT_ASSERT_EQUAL(std::string("http://proxy%40user@proxy:8080"),
+                       option.get(PREF_HTTP_PROXY));
+
+  option.put(PREF_HTTP_PROXY, "http://user@proxy:8080");
+  handler.parse(option, "proxyuser");
+  CPPUNIT_ASSERT_EQUAL(std::string("http://proxyuser@proxy:8080"),
+                       option.get(PREF_HTTP_PROXY));
+
+  option.put(PREF_HTTP_PROXY, "http://user:passwd%40@proxy:8080");
+  handler.parse(option, "proxyuser");
+  CPPUNIT_ASSERT_EQUAL(std::string("http://proxyuser:passwd%40@proxy:8080"),
+                       option.get(PREF_HTTP_PROXY));
+
+  handler.parse(option, "");
+  CPPUNIT_ASSERT_EQUAL(std::string("http://:passwd%40@proxy:8080"),
+                       option.get(PREF_HTTP_PROXY));
+}
+
+void OptionHandlerTest::testHttpProxyPasswdOptionHandler()
+{
+  HttpProxyPasswdOptionHandler handler(PREF_HTTP_PROXY_PASSWD, "", "");
+  Option option;
+  handler.parse(option, "proxypasswd");
+  CPPUNIT_ASSERT_EQUAL(std::string("proxypasswd"),
+                       option.get(PREF_HTTP_PROXY_PASSWD));
+
+  option.put(PREF_HTTP_PROXY, "http://proxy:8080");
+  handler.parse(option, "proxy@passwd");
+  CPPUNIT_ASSERT_EQUAL(std::string("proxy@passwd"),
+                       option.get(PREF_HTTP_PROXY_PASSWD));
+  CPPUNIT_ASSERT_EQUAL(std::string("http://:proxy%40passwd@proxy:8080"),
+                       option.get(PREF_HTTP_PROXY));
+
+  option.put(PREF_HTTP_PROXY, "http://:pass@proxy:8080");
+  handler.parse(option, "proxypasswd");
+  CPPUNIT_ASSERT_EQUAL(std::string("http://:proxypasswd@proxy:8080"),
+                       option.get(PREF_HTTP_PROXY));
+
+  option.put(PREF_HTTP_PROXY, "http://user%40:pass@proxy:8080");
+  handler.parse(option, "proxypasswd");
+  CPPUNIT_ASSERT_EQUAL(std::string("http://user%40:proxypasswd@proxy:8080"),
+                       option.get(PREF_HTTP_PROXY));
+
+  handler.parse(option, "");
+  CPPUNIT_ASSERT_EQUAL(std::string("http://user%40:@proxy:8080"),
+                       option.get(PREF_HTTP_PROXY));
+  
 }
 
 } // namespace aria2
